/* crc.c
   elementary crc functions: truncating/extending a buffer, 
   adding/checking a crc, and so on 
   
   Author : KoreK
   Date : Aug 3, 2004
   Licence : GPL v2 or above (see COPYING file or get it from the FSF)

   Copyright (c) 2004 KoreK, All Rights Reserved

   Warning: low-level functions do not check that the buffer has enough space 
   allocated to perform the operations, nor do they adjust the buffer length.
*/

#include <sys/types.h>
#include "crc-tables.h"
#include "crc.h"


/* low-level functions */

/* return crc of buffer (u_char *)buf of size (size_t)len */
u_int crc(const u_char *buf,size_t len)
{
  u_int rem=0xffffffffU;

  for (;len>0; len--,buf++)
      rem=crc_tbl[(rem ^ *buf) & 0xff] ^ (rem >> 8);
  return ~rem;
}


/* check crc validity of (u_char *)buf of size (size_t)len */
/* returns boolean value (0 for failure) */
u_int chk_crc(const u_char *buf,size_t len)
{

    u_int rem=0xffffffffU;
    u_int rem2=0;
    
    for (;len>4; len--,buf++)
	rem=crc_tbl[(rem ^ *buf) & 0xff] ^ (rem >> 8);
    rem=~rem;

    for (;len>0; len--,buf++) rem2=(rem2>>8)|(*buf<<24);
    return !(rem2^rem);
}


/* add  4-byte crc at the end of buffer buf of size len 
   buf must have at least len+4 byte allocated */
void add_crc(u_char *buf,size_t len)
{
    u_int c;
    c=crc(buf,len);
    buf[len++]=c & 0xff;
    c>>=8;
    buf[len++]=c & 0xff;
    c>>=8;
    buf[len++]=c & 0xff;
    c>>=8;
    buf[len++]=c & 0xff;
    return ;
}


/* assuming crc-valid buf has size len, buf[len-1]==a, 
   modify buf so that it becomes a crc-valid buffer of length len-1 */ 
void trunc_crc(u_char *buf,size_t len,u_char a)
{
    buf[--len]^=a;
    buf[--len]^=crc_chop_tbl[a][3];
    buf[--len]^=crc_chop_tbl[a][2];
    buf[--len]^=crc_chop_tbl[a][1];
    buf[--len]^=crc_chop_tbl[a][0];
}


/* "invert" the previous transformation :
   trunc_crc(buf,len,a);
   untrunc_crc(buf,len,a);
   leaves buf unmodified */
void untrunc_crc(u_char *buf,size_t len,u_char a)
{
    trunc_crc(buf,len+1,a);
    return ;
}


/* ext_crc1 and ext_crc2 extend crc-valid buffer buf with 1 byte */
void ext_crc1(u_char *buf,size_t len)
{
    buf[len--]=crc_ext[3];
    buf[len--]^=crc_ext[2];
    buf[len--]^=crc_ext[1];
    buf[len]^=crc_ext[0];
}

void ext_crc2(u_char *buf,size_t len)
{
    buf[len]=0xff;
    buf[len-4]^=0xff;
}


/* add a 4-byte crc_mask to buf of size len - 
   any crc-valid (len+4)-sized buffed xored with the obtained 
   (len+4)-sized buffer remains valid */ 
void add_crc_mask(u_char *buf,size_t len)
{
  u_int rem=0;
  
  for (;len>0; len--,buf++)
      rem=crc_tbl[rem & 0xff] ^ (rem >> 8) ^ (*buf << 24);
  
  for (len=0; len<4; len++) 
      rem=crc_tbl[rem & 0xff] ^ (rem >> 8);

  buf[0]=rem & 0xff;
  rem>>=8;
  buf[1]=rem & 0xff;
  rem>>=8;
  buf[2]=rem & 0xff;
  rem>>=8;
  buf[3]=rem & 0xff;
}

